#!/bin/sh

HA_PREFIX="/var/packages/HighAvailability/target"
#shellcheck source-path=SCRIPTDIR source=rc.subr
. $HA_PREFIX/etc/rc.subr
lsyncdInstance=${1:?"Missing instance name"}
SYNOLSYNC="$HA_PREFIX/sbin/synolsync"

interval=2
retryMax=60   #total wait (60 * interval) seconds
retry=0
while ! status=$($SYNOLSYNC --get-status "$lsyncdInstance"); do
	retry=$(( retry + 1 ))
	if [ $retry -gt $retryMax ]; then
		synoha_log warning "Timed-out on waiting lsyncd instance $lsyncdInstance"
		exit 1
	fi
	synoha_log notice "Lsyncd instance $lsyncdInstance is syncing files. Keep waiting: $retry/$retryMax"
	sleep $interval
done
hasErr=$(echo "$status" | sed -n 's/error:\(.*\)/\1/p')
isWait=$(echo "$status" | sed -n 's/wait:\(.*\)/\1/p')
hasRemote=$(HAHasRemote)

if [ 0 = "$hasErr" ] && [ 0 = "$isWait" ]; then
	while :;
	do
		if [ -z "$hasRemote" ]; then
			synoha_log notice "Remote is not online. Skip waiting lsyncd instance $lsyncdInstance"
			break
		elif synoha --is-safemode; then
			synoha_log notice "Split-brain occurred. Skip waiting lsyncd instance $lsyncdInstance"
			break
		fi
		if [ "0" != "$($SYNOLSYNC --get-events "$lsyncdInstance" | head -n 1)" ]; then
			retry=$(( retry + 1 ))
			if [ $retry -gt $retryMax ]; then
				synoha_log warning "Timed-out on waiting lsyncd instance $lsyncdInstance"
				break
			fi
			synoha_log notice "Lsyncd instance $lsyncdInstance is syncing files. Keep waiting: $retry/$retryMax"
			sleep $interval # delay interval seconds, it is lsyncd's event aggregation time
		else
			synoha_log notice "Lsyncd instance $lsyncdInstance is ready to stop"
			break
		fi
	done
fi

