#!/bin/sh
# Copyright (c) 2000-2012 Synology Inc. All rights reserved.

HA_PREFIX="/var/packages/HighAvailability/target"
PIDFILE="/var/run/ntpd_ha_client.pid"
NTPD_BIN="/usr/sbin/ntpd"
NTPCONF="/tmp/ntp_ha_client.conf"
HA_INFO="/usr/syno/etc/packages/HighAvailability/ha.conf"
DRBD_IF=`get_key_value $HA_INFO drbd_if`
HA_IF_MAIN=`get_key_value $HA_INFO ha_if_main`

. $HA_PREFIX/etc/rc.subr

start_()
{
	rm -f $NTPCONF
	local _remote_drbdip=`$SYNOHA_BIN --remote-drbdip`
	local _remote_ip=`$SYNOHA_BIN --remote-ip`
	local _drbd_node_0_ip=""
	local _drbd_node_1_ip=""
	if is_support_xa; then
		_remote_ip="$NODE_NTB_IP_REMOTE"
		_drbd_node_0_ip="169.254.3.1"
		_drbd_node_1_ip="169.254.3.2"
	else
		_drbd_node_0_ip="169.254.1.1"
		_drbd_node_1_ip="169.254.1.2"
	fi
	local _local_drbdip=""
	[ "$_drbd_node_0_ip" == "$_remote_drbdip" ] && _local_drbdip="$_drbd_node_1_ip" || _local_drbdip="$_drbd_node_0_ip"

	cat > $NTPCONF <<EOF
server $_remote_drbdip prefer
server $_remote_ip
restrict default ignore
restrict -6 default ignore
restrict 127.0.0.1
restrict $_remote_drbdip
restrict $_remote_ip
interface ignore wildcard
interface listen $DRBD_IF
interface listen $_local_drbdip
EOF
	if ! is_support_xa; then
		echo "interface listen $HA_IF_MAIN" >> $NTPCONF
	fi

	$NTPD_BIN -p $PIDFILE -c $NTPCONF -g -4 -u ntp:ntp
}

stop_()
{
	if [ -f $PIDFILE ]; then
		local _pid
		_pid=`cat $PIDFILE`
		if [ "x$_pid" != "x" ] ; then
			kill $_pid
		fi
		rm -f $NTPCONF
		rm -f $PIDFILE
	fi
}

case "$1" in
	start)
		stop_
		start_
		;;
	stop)
		stop_
		;;
	restart)
		stop_
		start_
		;;
	*)
		echo "Usage: $0 { start | stop | restart}"
		;;
esac
exit $?

