#!/bin/sh

HA_PREFIX="/var/packages/HighAvailability/target"
HA_BIN=$HA_PREFIX"/sbin/synoha"

. $HA_PREFIX/etc/rc.subr
. $HA_PREFIX/etc/rc.synologset

SZ_CHK_SAFEMODE_REPLY_TRUE="${AIS_CHECK_SAFEMODE_REPLY}_true__"
SZ_CHK_SAFEMODE_REPLY_FALSE="${AIS_CHECK_SAFEMODE_REPLY}_false_"
SZF_SYNO_VERSION_DEF="/etc.defaults/VERSION"
SZF_SYNO_VERSION_DEF_TMP="${SZF_SYNO_VERSION_DEF}.tmp"

log()
{
	(
		flock -x -w 10 9
		echo "$(date '+%Y-%m-%dT%H:%M:%S%:z') $*" >> /var/log/cluster/ais.action
	) 9>/var/run/ha/ais.action.lock
}

update_ha_config_with_model_info()
{
	# Please delete after next version
	get_bin=/usr/syno/bin/synogetkeyvalue
	set_bin=/usr/syno/bin/synosetkeyvalue
	conf=/tmp/synoinfo.conf
	ha_conf=/var/packages/HighAvailability/etc/ha.conf
	local_name=$($HA_BIN --local-name)
	lnode_name=$($get_bin $ha_conf host0)
	index=0
	if [ "x$local_name" = "x$lnode_name" ]; then
		index=1
	fi

	if ! /usr/bin/rsync --port=874 -acs \
		--password-file=/usr/syno/etc/packages/HighAvailability/confsync/rsync.pw \
		root@"$DRBD_IP_REMOTE"::synoha_root/etc.defaults/synoinfo.conf "$conf"; then
		log "Failed to retrieve remtoe's synoinfo.conf"
	fi
	unique=$($get_bin $conf unique)
	unique=${unique#synology_}
	maxdisks=$($get_bin $conf maxdisks)
	$set_bin $ha_conf model$index "$unique"
	$set_bin $ha_conf maxdisks$index "$maxdisks"
	/bin/grep -v corosync_key $ha_conf > /tmp/.ha.conf.new
	mv /tmp/.ha.conf.new /tmp/.ha.conf
	log "Update ha.conf with model info done"
}

if [ $# != 1 ] || [ "/tmp/.ais." != "$(echo "$1" | head -c 10)" ]; then
	log "unknown file: $*"
	exit 1
fi

# $1 is a tmp file (name)
# first line is the message/header/command, others are the file content (if any)
# 1-8 bytes: header
header=$(head -c 8 "$1")
case "$header" in
	"STRUCT  ")
		log "[STRUCT] $1"
		{
			$HA_PREFIX/sbin/synodrbd --handle-ais-msg "$1"
			rm -f "$1"
		}&
		;;
#todo: handle reboot by cgi or something
	$AIS_REBOOT)
		log "[Reboot]"
		{
			sleep 3
			$HA_PREFIX/etc/rc.ha stop
			reboot
			rm -f "$1"
		}&
		;;
#todo: handle poweroff by cgi or something
	$SYNO_HA_AIS_POWEROFF)
		log "[Poweroff]"
		{
			sleep 3
			$HA_BIN --poweroff-ds
			rm -f "$1"
		}&
		;;
	$AIS_SYNOLOG_REQ)
		#log "[LogReq]"
		{
			$HA_BIN --receive-synolog-info "$1"
			rm -f "$1"
		}&
		;;
	$AIS_DO_SCRUBBING)
		{
			data=$(cat "$1")
			! is_support_xa && $HA_BIN --receive-do-scrubbing "$data"
			rm -f "$1"
		}&
		;;
	$AIS_CANCEL_SCRUBBING)
		{
			data=$(cat "$1")
			! is_support_xa && $HA_BIN --receive-cancel-scrubbing "$data"
			rm -f "$1"
		}&
		;;
	$AIS_PAUSE_SCRUBBING)
		{
			data=$(cat "$1")
			! is_support_xa && $HA_BIN --receive-pause-scrubbing "$data"
			rm -f "$1"
		}&
		;;
	$AIS_SYNOLOG_RESP)
		#log "[LogResp]"
		cut -b 9-16 "$1" > "$HA_LOG_RESP_RESULT"
		rm -f "$1"
		;;
	$AIS_INFOSET_REQ)
		#log "[InfosetReq]"
		type=$(cut -b 9 "$1");
		(
			if flock -n -x 9
			then
				${HA_SBIN_DIR}/synohainfoset --receive-info "$1"
			fi
			rm -f "$1"
		) 9>/tmp/.ha."$type" &
		;;
	"UNBIND  ")
		log "[Unbind]"
		{
			sleep 3
			unbind_type=$(cut -b 9-1024 "$1")
			# This form for POSIX standard
			$HA_PREFIX/sbin/synoha --unbind-local $UNBIND_WITH_REBOOT "$unbind_type" > /dev/null 2>&1 &
			# wait reboot after unbind local
			sleep 300
			reboot -f
			rm -f "$1"
		}&
		;;
	"UB_REMOT")
		log "[Unbind Remote]"
		{
			$HA_PREFIX/sbin/synoha --unbind-remote &
			rm -f "$1"
		}&
		;;
	"SAVE_KEY")
		log "[Save_key]"
		{
			cat "$1" > "$SYNO_HA_AUTH_KEY"
			$HA_PREFIX/sbin/synoha --auth-key "$(cat "$1")"
			rm -f "$1"
		}&
		;;
	"SHUTBEEP")
		log "[Shut_Beep]"
		{
			$HA_PREFIX/sbin/synoha --stop-beep
			rm -f "$1"
		}&
		;;
	"CONF_CHG")
		log "[SYNOINFO_CHANGE]"
		{
			$HA_PREFIX/sbin/synoha --signal-scemd synoinfo-change
			rm -f "$1"
		}&
		;;
	"FENCING ")
		{
			action=$(cut -b 9-16 "$1")
			log "[FENCING] $action"
			case "$action" in
				"TRYFENCE")
					handle_too_active "$action"
					;;
				"DONE    ")
					touch "$FLAG_HA_REMOTE_IS_FENCED"
					;;
				*)
					synoha_log "unknown action: ${action}."
					;;
			esac
			rm -f "$1"
		}&
		;;
	"UPGRADE ")
		# the same as ha_upgrade.cc
		# MUST change them both
		log "[Upgrade]"
		HA_UPGRADE_INFO_TMP="$HA_UPGRADE_INFO.tmp"
		SZK_HA_UPG_PROGRESS="progress"
		# Reference the struct HA_AIS_UPG_HEADER with member "mark", "cmd", and "data"
		cmd=$(cut -b 9-16 "$1")
		data=$(cut -b 17-80 "$1")
		case "$cmd" in
			"DONE_ACT"|"FAIL_ACT")
				log "[upg result $cmd]"
				if [ "$cmd" = "ERR_PASS" ] && [ -n "$data" ]; then
					echo "$data" > "/tmp/update.progress"
				fi
				sed "/$SZK_HA_UPG_PROGRESS/d" "$HA_UPGRADE_INFO" > "$HA_UPGRADE_INFO_TMP"
				if [ -s "$HA_UPGRADE_INFO_TMP" ]; then
					echo "$SZK_HA_UPG_PROGRESS=$cmd" >> "$HA_UPGRADE_INFO_TMP"
					mv "$HA_UPGRADE_INFO_TMP" "$HA_UPGRADE_INFO"
				fi
				;;
			"FAIL_UPG")
				log "[upg fail $cmd]"
				rm "$HA_UPGRADE_INFO"
				/sbin/initctl stop pkg-synoha-upgrade-passive-prestart
				;;
			"DONE_PAS"|"FAIL_PAS")
				log "[upg result $cmd]"
				if ! is_support_xa || $HA_BIN --is-ha-upgrading; then
					touch "$HA_REMOTE_UPGRADE"
					flock -x "$HA_REMOTE_UPGRADE_LOCK" synosetkeyvalue "$HA_REMOTE_UPGRADE" "$SZK_HA_UPG_PROGRESS" "$cmd"
				fi

				if is_support_xa && [ "DONE_PAS" = "$cmd" ]; then
					# SHA #2253: update active net conf on passive
					synoha_log notice "Rsync [$SZD_ACTIVE_NET_CONF] to remote"
					rsync_to_remote "$SZD_ACTIVE_NET_CONF/"
				fi

				update_ha_config_with_model_info
				gen_local_ip_list
				send_local_ip_list

				# FIXME synchronize hotspare.conf to avoid bugs due to empty hotspare.conf
				touch /usr/syno/etc/hotspare.conf
				;;
			"PRESTART")
				log "[passive upg pre-start]"
				$HA_BIN --upg-passive-pre-start
				;;
			"START   ")
				log "[upg start]"
				# This is the passive upgrade entry point
				/sbin/initctl stop pkg-synoha-upgrade-passive-prestart
				$HA_BIN --upg-start-passive &
				;;
			"STARTCU ")
				log "[upg start cu]"
				# This is the passive upgrade entry point
				/sbin/initctl stop pkg-synoha-upgrade-passive-prestart
				$HA_BIN --upg-start-cu-passive &
				;;
			"STARTPKG")
				log "[upg pkg start, will restart all services]"
				# This is the passive upgrade entry point
				/sbin/initctl stop pkg-synoha-upgrade-passive-prestart
				$HA_BIN --upg-pkg-start-passive &
				;;
			*)
			log "unknown cmd: $cmd"
				;;
		esac
		rm -f "$1"
		;;
	$SZ_HA_NODE_ONLINE)
		log "[Node online]"
		{
			_host=$(cut -b 9-48 "$1")
			log "node $_host online"
			if [ $ROLE_ACTIVE = "$($HA_BIN --local-role)" ]; then
				if [ -f "$FLAG_HA_REMOTE_IS_FENCED" ] ; then
					rm -f "$FLAG_HA_REMOTE_IS_FENCED"
					synoha_log notice "clean up remove fencing flag"
				fi

				HAWaitCibInfoReady
				! is_support_xa && $HA_BIN --check-remote-ssd-cache > /dev/null 2>&1
				$HA_BIN --check-memsize-when-cache-exist > /dev/null 2>&1
				! is_support_xa && $HA_BIN --check-flashcache-sysctl > /dev/null 2>&1
				$HA_BIN --check-ver-mismatch > /dev/null 2>&1
				$HA_BIN --refresh-user-action-impact on-passive-online > /dev/null 2>&1
				$HA_BIN --notify passive-boot-done "$REMOTE_HOST" > /dev/null 2>&1
				$HA_BIN --save-ha-state-time active
			fi

			rm -f "$1"
			rm -f "$REMOTE_SHUTDOWN_FLAG"
			rm -f "$REMOTE_REBOOTING_FLAG"
			rm -f "$WAIT_FSCK_FLAG"
			rm -f /tmp/ha/no_active_shutdown # See Checkpoint_last
			rm -f /tmp/ha/no_active_cnt # See Checkpoint_last
			rm -f /tmp/ha/ais_communication # Reset ais communication test

			#FIXME define a variable for this flag
			rm -f "/tmp/ha/.ha_waiting_remote_online_ais_flag"
			rm -f "$PASSIVE_IS_REBOOTING_TRIGGERED_BY_WEBAPI"

			gen_local_ip_list
			send_local_ip_list
		}&
		;;
	$AIS_PASSIVE_MAC)
		log "[Recive mac form remote node]"
		{
			$HA_PREFIX/sbin/synoha --fill-ha-mac-passive "$1"
			rm -f "$1"
		}&
		;;
	$AIS_RUN_SSHD)
		log "[Run sshd]"
		{
			/usr/syno/sbin/synoservice --resume-by-reason ssh-shell ha-passive
			rm -f "$1"
		}&
		;;
	$AIS_STOP_SSHD)
		log "[Stop sshd]"
		{
			/usr/syno/sbin/synoservice --pause-by-reason ssh-shell ha-passive
			rm -f "$1"
		}&
		;;
	$AIS_HANDLE_SYNOSERVICE)
		{
			log "$(cat "$1")"
			action=$(cut -b 9-16 "$1" | tr -d ' ')
			service=$(cut -b 17-32 "$1" | tr -d ' ')
			log "[$action $service]"
			log "/usr/syno/sbin/synoservice --$action $service"
			/usr/syno/sbin/synoservice --"$action" "$service"
			rm -f "$1"
		}&
		;;
	$AIS_RUN_SAFEMODE)
		log "[Run safemode]"
		{
			if [ ! -f $HA_SAFEMODE_INFO ]; then
				$HA_PREFIX/sbin/handler_sb.sh safemode
			fi
			rm -f "$1"
		}&
		;;
	$AIS_CHECK_SAFEMODE)
		log "[Check safemode]"
		{
			if [ -f $HA_SAFEMODE_INFO ]; then
				crm_node --node="$REMOTE_HOST" --mesg="${SZ_CHK_SAFEMODE_REPLY_TRUE}"
			else
				crm_node --node="$REMOTE_HOST" --mesg="${SZ_CHK_SAFEMODE_REPLY_FALSE}"
			fi
			rm -f "$1"
		}&
		;;
	$AIS_CHECK_SAFEMODE_REPLY)
		log "[Check safemode reply]"
		{
			rm -f $SB_REMOTE_IN_SAFEMODE $SB_REMOTE_NOT_IN_SAFEMODE
			if [ "$(head -c 15 "$1")" = "$SZ_CHK_SAFEMODE_REPLY_TRUE" ]; then
				touch $SB_REMOTE_IN_SAFEMODE
			else
				touch $SB_REMOTE_NOT_IN_SAFEMODE
			fi
			rm -f "$1"
		}&
		;;
	$AIS_SB_SET_ROLE)
		{
			data=$(cut -b 17-80 "$1")
			log "[Set SB Role:$data]"
			eval "$HA_BIN --set-sb-role $data"
			rm -f "$1"
		}&
		;;
	$AIS_CHANGE_DSM_VERSION)
		log "[Change dsm version] $1"
		{
			version=$(cut -b 9-24 "$1");
			major=$(echo "$version" | cut -d"," -f 1)
			minor=$(echo"$version" | cut -d"," -f 2)
			build=$(echo "$version" | cut -d"," -f 3)
			log "major=$major minor=$minor build=$build"
			sed 's/^majorversion=.*$/majorversion="'"$major"'"/; s/^minorversion=.*$/minorversion="'"$minor"'"/; s/^buildnumber=.*$/buildnumber="'"$build"'"/' $SZF_SYNO_VERSION_DEF > $SZF_SYNO_VERSION_DEF_TMP
			mv $SZF_SYNO_VERSION_DEF_TMP $SZF_SYNO_VERSION_DEF
			rm -f "$1"
		}&
		;;
	$AIS_NOTIFY_REMOTE)
		log "[Notification from remote]"
		{
			data="Notification from $REMOTE_HOST - [$(cut -b 9-520 "$1")]"
			synoha_log warning "$data"
			rm -f "$1"
		}&
		;;
	$AIS_TOUCH_SHUTDOWN_FLAG)
		log "[Remote shutdown first]"
		{
			touch "$REMOTE_SHUTDOWN_FLAG"
			rm -f "$1"
		}&
		;;
	$AIS_TOUCH_WAIT_FSCK_FLAG)
		log "[Do fsck now]"
		{
			touch "$WAIT_FSCK_FLAG"
			rm -f "$1"
		}&
		;;
	$AIS_TOUCH_REMOTE_REBOOTING_FLAG)
		log "[Touch remote rebooting flag]"
		{
			touch "$REMOTE_REBOOTING_FLAG"
			rm -f "$1"
		}&
		;;
	$AIS_SYNOLOG_REMOTE)
		log "[Get synolog]"
		{
			args=$(cut -b 9-264 "$1")
			eval "$SYNOLOGSET1_BIN $args"
			rm -f "$1"
		}&
		;;
	$AIS_SYNC_MD0_MD1)
		log "[Sync md0 partion]"
		{
			# wait for conf sync done
			sleep 5
			synostgsysraid --sync
			rm -f "$1"
		}&
		;;
	$AIS_PASSIVE_ENTER_UPS_SAFEMODE)
		log "[Passive enter UPS safe mode]"
		synoha_log notice "Standby passive server before entering UPS safe mode"
		$SYNOHA_BIN --standby-local
		{
			[ -e "$HA_IN_UPS_SAFEMODE_FLAG" ] && exit 0
			touch "$HA_IN_UPS_SAFEMODE_FLAG"
			sleep 3
			enter_ups_safemode
			rm -f "$1"
		}&
		;;
	"API_REQ ")
		log "[Exec webapi]"
		{
			pid=$(cut -b 9-16 "$1" | tr -d " ")
			args=$(cut -b 17-512 "$1")
			echo "API_RESP" > /tmp/ha/.ha.webapi.$$
			echo "$pid" >> /tmp/ha/.ha.webapi.$$
			eval synowebapi "$args" >> /tmp/ha/.ha.webapi.$$
			crm_node --node="$REMOTE_HOST" --mesg="-fIlE-:/tmp/ha/.ha.webapi.$$"

			rm -f /tmp/ha/.ha.webapi.$$
			rm -f "$1"
		}&
		;;
	"API_RESP")
		log "[response webapi]"
		{
			sed -i '1d' "$1"
			pid=$(head -n 1 "$1")
			sed -i '1d' "$1"
			cp "$1" /tmp/ha/ha.webapi.return."$pid"

			rm -f "$1"
		}&
		;;
	"COM_TEST")
		# Do not record because it may be stucked by hard disk IO.
		# log "[Communication test]"
		{
			pid=$(cut -b 9-16 "$1" | tr -d " ")
			target_file=/tmp/ha/.communication.$$
			echo "COM_RESP" > $target_file
			echo "$pid" >> $target_file
			crm_node --node="$REMOTE_HOST" --mesg="-fIlE-:$target_file"

			rm -f $target_file
			rm -f "$1"
		}&
		;;
	"COM_RESP")
		# Do not record because it may be stucked by hard disk IO.
		# log "[Response communication test]"
		{
			sed -i '1d' "$1"
			pid=$(head -n 1 "$1")
			cp "$1" /tmp/ha/.communication.return."$pid"

			rm -f "$1"
		}&
		;;
	"P_CHKREQ")
		log "[check patch/space]"
		{
			pid=$(cut -b 9-16 "$1" | tr -d " ")
			patch_path=$(cut -b 17-512 "$1")
			resp_file=/tmp/ha/.ha.patch.check.$$
			ret=0

			echo "P_CHKRES" > $resp_file
			echo "$pid" >> $resp_file

			response=$(synowebapi --exec api=SYNO.Core.Upgrade method=status 2>/dev/null | jq -c '.' -)
			allow_upgrade=$(echo $response | jq -c '.data.allow_upgrade')
			if [ "true" != "$allow_upgrade" ]; then
				synoha_log error "$response"
				if echo "$response" | grep "no enough volume space"; then
					/bin/df -h | synoha_log notice
					ret=2
				else
					ret=1
				fi
			else
				response=$(/usr/syno/sbin/synoupgrade --check-pat "$patch_path")
				if [ "0" != "$?" ]; then
					synoha_log notice "check patch failed"
					synoha_log notice "$response"
					ret=3
				fi
			fi

			if [ "0" != $ret ]; then
				rm -f "$patch_path"
			fi
			printf "%s" "$ret" >> $resp_file
			crm_node --node="$REMOTE_HOST" --mesg="-fIlE-:$resp_file"

			rm -f $resp_file
			rm -f "$1"
		}&
		;;
	"P_CHKRES")
		log "[response check patch/space]"
		{
			sed -i '1d' "$1"
			pid=$(head -n 1 "$1")
			sed -i '1d' "$1"
			cp "$1" /tmp/ha/ha.patch.check.return."$pid"

			rm -f "$1"
		}&
		;;
	"PKG_CHKQ")
		log "[check pkg]"
		{
			pid=$(cut -b 9-16 "$1" | tr -d " ")
			pkg_path=$(cut -b 17-512 "$1")
			resp_file=/tmp/ha/.ha.pkg.check.$$

			echo "PKG_CHKS" > $resp_file
			echo "$pid" >> $resp_file
			$HA_BIN --check-pkg "$pkg_path" > /dev/null 2>&1
			ret=$?
			if [ "$ret" != "0" ]; then
				synoha_log notice "synoha --check-pkg $pkg_path failed: $ret"
				/bin/df -h | synoha_log notice
				rm -f "$pkg_path"
			fi
			printf "%s" "$ret" >> $resp_file
			crm_node --node="$REMOTE_HOST" --mesg="-fIlE-:$resp_file"

			rm -f $resp_file
			rm -f "$1"
		}&
		;;
	"PKG_CHKS")
		log "[response check pkg]"
		{
			sed -i '1d' "$1"
			pid=$(head -n 1 "$1")
			sed -i '1d' "$1"
			cp "$1" /tmp/ha/ha.pkg.check.return."$pid"

			rm -f "$1"
		}&
		;;
	"SSD_TRIM")
		log "[trim ssd]"
		{
			space=$(cut -b 9-136 "$1")
			/usr/syno/bin/syno_ssd_trim "$space" > /dev/null 2>&1

			rm -f "$1"
		}&
		;;
	"PINGRECV")
		log "[Remote received bumped admin_epoch]"
		{
			touch "$SYNO_HA_REMOTE_PING_READY"
			rm -f "$1"
		}&
		;;
	"SMARTEST")
		log "[do smart test]"
		{
			arg=$(cut -b 9-1032 "$1")
			/usr/syno/bin/syno_smart_test "$arg" > /dev/null 2>&1

			rm -f "$1"
		}&
		;;
	$SZ_HA_SYNC_IP)
		log "[sync ip]"
		{
			sed -i '1d' "$1"
			if [ ! -f "$SZF_REMOTE_IP_LIST" ]; then
				mv "$1" "$SZF_REMOTE_IP_LIST"
				gen_local_ip_list
				send_local_ip_list
			else
				mv "$1" "$SZF_REMOTE_IP_LIST"
			fi
			rm -f "$1"
		}&
		;;
	"LOGSET  ")
		log "[synologset]"
		{
			line=$(sed -n '2p' "$1")
			if ! evtID=$(get_event "$line"); then
				synoha_log notice "unknown severity"
				exit
			fi

			if ! appID=$(get_app_for_remote); then
				synoha_log notice "unknown app"
				exit
			fi

			if ! msg=$(get_message "$line"); then
				synoha_log notice "unknown msg"
				exit
			fi

			$SYNOLOGSET_BIN "$appID" "$evtID" "$msg"
			rm -f "$1"
		}
		;;
	"SAVESTAT")
		log "[update time stat]"
		{
			sed -i '1d' "$1"
			remote_state=$(cut -b 1-512 "$1")
			local_file_path=/var/lib/ha/ha.state.time.info

			remote=$([ "x$REMOTE_HOST" = "x$NODE_HOST0" ] && echo host0 || echo host1)
			time=$(echo "$remote_state" | jq -r .sys_info."$remote"_active_time)
			filter=$(printf '.sys_info["%s_active_time"] = "%s"' "$remote" "$time")
			jq -c "$filter" $local_file_path > /tmp/ha.state.$$ && mv /tmp/ha.state.$$ $local_file_path

			rm -f "$1"
		}
		;;
	"GET_INFO")
		# Only for XA
		log "[Get info for remote]"
		{
			pid=$(cut -b 9-16 "$1" | tr -d " ")
			resp_file=/tmp/ha/ha.get.info.$$

			echo "INFO_RES" > $resp_file
			echo "$pid" >> $resp_file
			$HA_BIN --get-info $resp_file
			crm_node --node="$REMOTE_HOST" --mesg="-fIlE-:$resp_file"
			rm -f $resp_file
			rm -f "$1"
		}&
		;;
	"INFO_RES")
		# Only for XA
		log "[Get info from remote]"
		{
			sed -i '1d' "$1"
			pid=$(head -n 1 "$1")
			sed -i '1d' "$1"
			cp "$1" /tmp/ha/ha.get.info.return."$pid"

			rm -f "$1"
		}&
		;;
	"INITXAOK")
		# Only for XA
		log "[XA finish init with remote]"
		{
			rm -f /var/lib/ha/init_xa_with_remote
		}&
		;;
	"FIN_BIND")
		log "[Finish binding]"
		{
			rm -f "$DEACTIVATING"
			rm -f "$1"
		}&
		;;
	"SET_OVS ")
		{
			set_ovs_params=$(cut -b 9-1032 "$1" | tr -d " ")
			log "[Set OVS with params $set_ovs_params]"
			$HA_BIN --set-ovs "$set_ovs_params"
		}&
		;;
	"STOP_NET")
		log "[Stop network]"
		{
			stop_network
		}&
		;;
	*)
		log "unknown header: ${header}."
		rm -f "$1"
		exit 1
		;;
esac
exit 0
